@@ -22,7 +22,7 @@
 #include <gnome.h>
 #include <panel-applet.h>
 
-#define MAX_EYES 5
+#define MAX_EYES 1000
 typedef struct
 {
 	GtkWidget *pbox;
@@ -37,7 +37,7 @@ typedef struct
 	PanelApplet *applet;
 	GtkWidget   *vbox;
 	GtkWidget   *hbox;
-	GtkWidget   *eyes[MAX_EYES];
+	GtkWidget   **eyes;
 	guint        timeout_id;
 	gint pointer_last_x;
 	gint pointer_last_y;
@@ -70,6 +70,8 @@ void destroy_eyes (EyesApplet *eyes_applet);
 
 
 /* theme.c */
+void theme_dirs_create ();
+
 int load_theme    (EyesApplet        *eyes_applet,
 		   const gchar       *theme_dir);
 
