@@ -717,6 +717,9 @@ static gint updateOutput(gpointer data)
 		const gchar *translator_credits = _("translator_credits");
 
 		if (stockdata->about_dialog) {
+			gtk_window_set_screen (GTK_WINDOW (stockdata->about_dialog),
+					       gtk_widget_get_screen (stockdata->applet));
+
 			gtk_window_present (GTK_WINDOW (stockdata->about_dialog));
 			return;
 		}
@@ -1684,7 +1687,7 @@ static gint updateOutput(gpointer data)
     	{
     		StockData *stockdata = data;
     		
-    		if (stockdata->props.use_default_theme);
+    		if (stockdata->props.use_default_theme)
     			load_fonts (stockdata);
     	}
 
